<?php

/**
 * Menu callback; mojeID admin settings form.
 */
function mojeid_admin_settings($form_state) {
  module_load_include('inc', 'mojeid');

  $form = array();

  $form['mojeid_supress_openid'] = array(
    '#type' => 'checkbox',
    '#title' => t('Supress OpenID'),
    '#description' => t('Suppress general usage of OpenID on this site, only mojeID will be allowed.'),
    '#default_value' => variable_get('mojeid_supress_openid', FALSE),
  );

  $form['mojeid_login_method'] = array(
    '#type' => 'radios',
    '#title' => t('Login method'),
    '#options' => array(
      'username' => t('Require username'),
      'direct' => t('Direct login'),
    ),
    '#default_value' => variable_get('mojeid_login_method', 'username'),
    '#description' => t('Selects mojeID login method for the login form. The <em>Direct login</em> method will display just login button without any input field. The <em>Require username</em> method will display standard username input field.'),
  );

  $form['mojeid_direct_login_button'] = array(
    '#type' => 'radios',
    '#title' => t('Direct login button'),
    '#options' => array(
      'basic' => t('Basic form button'),
      'image' => t('Image button'),
    ),
    '#default_value' => variable_get('mojeid_direct_login_button', 'basic'),
  );

  $form['mojeid_promo_icons'] = array(
    '#type' => 'checkboxes',
    '#title' => t('Promote mojeID'),
    '#options' => array(
      'user_login_block' => t('User login block'),
      'user_login' => t('User login form'),
      'user_register' => t('User register form'),
      'openid_user_add' => t('OpenID form under user account'),
      'mojeid_benefits_page' => t('mojeID benefits page'),
    ),
    '#default_value' => variable_get('mojeid_promo_icons', array()),
    '#description' => t('Display Powered by mojeID or mojeID Login banners on the checked forms.'),
  );

  $form['motivation_program'] = array(
    '#type' => 'fieldset',
    '#title' => t('Motivation programme'),
  );

  $form['motivation_program']['mojeid_motivation_program'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable motivation programme'),
    '#description' => t('Enable motivation program. See this link for details: <a href="@url">Incentive programme for web service providers</a>.', array('@url' => 'http://www.mojeid.cz/page/877/')),
    '#default_value' => variable_get('mojeid_motivation_program', TRUE),
  );

  $form['motivation_program']['mojeid_realm'] = array(
    '#type' => 'textfield',
    '#title' => t('Realm'),
    '#description' => t('Set realm URL (including http:// or https://) which will identify you when user is registering using your site. Usually same as the website URL e.g.: http://www.example.com.'),
    '#default_value' => variable_get('mojeid_realm', ''),
  );

  $form['motivation_program']['mojeid_motivation_page_title'] = array(
    '#type' => 'textfield',
    '#title' => t('Motivation page title'),
    '#default_value' => variable_get('mojeid_motivation_page_title', _mojeid_motivation_page_defaults('title')),
  );

  $form['motivation_program']['mojeid_motivation_page_path'] = array(
    '#type' => 'textfield',
    '#title' => t('Motivation page path'),
    '#default_value' => variable_get('mojeid_motivation_page_path', _mojeid_motivation_page_defaults('path')),
  );

  $form['motivation_program']['mojeid_motivation_page_body'] = array(
    '#type' => 'textarea',
    '#title' => t('Motivation page body'),
    '#rows' => 10,
    '#default_value' => variable_get('mojeid_motivation_page_body', _mojeid_motivation_page_defaults('body')),
  );

  $form['motivation_program']['mojeid_motivation_page_body_format'] = filter_form(variable_get('mojeid_motivation_page_body_format', FILTER_FORMAT_DEFAULT));

  if (module_exists('openid_client_ax')) {
    $schema = openid_client_ax_schema_definitions();
    $supported_elements = mojeid_supported_elements();
    foreach ($supported_elements as $uri) {
      if (isset($schema[$uri])) {
        $short_name = openid_client_ax_attribute_name($uri);
        $section = $schema[$uri]['section'];
        $options[$section][$short_name] = $schema[$uri]['label'] . ' <small>(' . $uri . ')</small>';
      }
      else {
        drupal_set_message(t("Element %uri should be supported but it wasn't found in the schema definition.", array('%uri' => $uri)), 'warning');
      }
    }

    $form['advanced'] = array(
      '#type' => 'fieldset',
      '#title' => t('Advanced settings'),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );

    $form['advanced']['mojeid_required'] = array(
      '#type' => 'fieldset',
      '#title' => t('mojeID required attributes'),
      '#description' => t('Checked mojeID attributes will be required when user is loging-in. Do it only if you really need it.'),
      '#tree' => TRUE
    );

    $required_defaults = variable_get('mojeid_required', array());
    foreach ($options as $section => $section_options) {
      $form['advanced']['mojeid_required'][] = array(
        '#type' => 'checkboxes',
        '#title' => t('Section @section', array('@section' => $section)),
        '#options' => $section_options,
        '#default_value' => array_intersect_key($required_defaults, $section_options),
      );
    }
  }

  $form['#submit'][] = 'mojeid_admin_settings_submit';

  return system_settings_form($form);
}

/**
 * Validation handler for the mojeID admin settings form.
 *
 * @see mojeid_admin_settings()
 */
function mojeid_admin_settings_validate($form, &$form_state) {
  $values =& $form_state['values'];

  if (!empty($values['mojeid_motivation_program'])) {
    if (isset($values['mojeid_realm']) && !valid_url($values['mojeid_realm'], TRUE)) {
      form_set_error('mojeid_realm', t("When motivation programme is enabled, realm must contain valid URL."));
    }
    if (empty($values['mojeid_motivation_page_title'])) {
      form_set_error('mojeid_motivation_page_title', t("When motivation programme is enabled, page title is required."));
    }
    if (empty($values['mojeid_motivation_page_path'])) {
      form_set_error('mojeid_motivation_page_path', t("When motivation programme is enabled, page path is required."));
    }
  }
}


/**
 * Submit handler for the mojeID admin settings form.
 *
 * @see mojeid_admin_settings()
 */
function mojeid_admin_settings_submit($form, &$form_state) {
  $values =& $form_state['values'];

  // Flatten AX schema required settings checkboxes.
  if (isset($values['mojeid_required']) && is_array($values['mojeid_required'])) {
    $flattened = array();
    foreach ($values['mojeid_required'] as $section) {
      $flattened += $section;
    }
    $values['mojeid_required'] = $flattened;
  }

  // Rebuild menu when some important setting is changed.
  if ($values['mojeid_motivation_program'] != variable_get('mojeid_motivation_program', TRUE)
      || $values['mojeid_motivation_page_title'] != variable_get('mojeid_motivation_page_title', _mojeid_motivation_page_defaults('title'))
      || $values['mojeid_motivation_page_path'] != variable_get('mojeid_motivation_page_path', _mojeid_motivation_page_defaults('path'))
  ) {
    // We have to change settings before we rebuild the menu.
    variable_set('mojeid_motivation_program', $values['mojeid_motivation_program']);
    variable_set('mojeid_motivation_page_title', $values['mojeid_motivation_page_title']);
    variable_set('mojeid_motivation_page_path', $values['mojeid_motivation_page_path']);
    menu_cache_clear_all();
  }
}
